import math;
import hyperbolic_geometry;

size(500,500);
settings.outformat="pdf";

// Here we show the construction of the circumcircle of a triangle.
// This is done in analogy to the classical euclidean geometry.

hyperbolic_point O = hyperbolic_point(0,0);
dot(O,red);

// The two points A, B given
hyperbolic_point A = hyperbolic_point(1.4,330);
hyperbolic_point B = hyperbolic_point(2.2,60);
hyperbolic_point C = hyperbolic_point(1,190);

// Triangle without circumcircle:
// hyperbolic_point A = hyperbolic_point(0.4,350);
// hyperbolic_point B = hyperbolic_point(2.2,60);
// hyperbolic_point C = hyperbolic_point(1,190);

// The triangle
hyperbolic_triangle T = hyperbolic_triangle(A,B,C);

//------------------------------------------------------------------------------
// Construct the midpoint of side AB

// The side L1 connection the A, B
hyperbolic_line L1 = hyperbolic_line(A,B);

// Two circles with center {A,B} going through {B,A}
hyperbolic_circle C1 = hyperbolic_circle(A,B);
hyperbolic_circle C2 = hyperbolic_circle(B,A);

// Intersect the two circles, resulting in the points I1, I2
hyperbolic_point[] I = intersection(C1,C2);

// Put a line L through the two intersection points I1,I2
hyperbolic_line SL1 = hyperbolic_line(I[0], I[1]);

// Intersect the line L with the segment S resulting in the midpoint M
hyperbolic_point MAB = intersection(SL1, L1);

// And draw all the stuff
draw(C1, gray+opacity(0.3));
draw(C2, gray+opacity(0.3));
draw(L1, gray+opacity(0.3));
dot(I[0], gray);
dot(I[1], gray);

//------------------------------------------------------------------------------
// Construct the midpoint of side AC
// The side L2 connection the A, C
hyperbolic_line L2 = hyperbolic_line(A,C);

// Two circles with center {A,C} going through {C,A}
hyperbolic_circle C1 = hyperbolic_circle(A,C);
hyperbolic_circle C2 = hyperbolic_circle(C,A);

// Intersect the two circles, resulting in the points I1, I2
hyperbolic_point[] I = intersection(C1,C2);

// Put a line L through the two intersection points I1,I2
hyperbolic_line SL2 = hyperbolic_line(I[0], I[1]);

// Intersect the line L with the segment S resulting in the midpoint M
hyperbolic_point MAC = intersection(SL2, L2);

// And draw all the stuff
draw(C1, gray+opacity(0.3));
draw(C2, gray+opacity(0.3));
draw(L2, gray+opacity(0.3));
dot(I[0], gray);
dot(I[1], gray);

//------------------------------------------------------------------------------
// Construct the midpoint of side BC
// The side L3 connection the B, C
hyperbolic_line L3 = hyperbolic_line(B,C);

// Two circles with center {B,C} going through {C,B}
hyperbolic_circle C1 = hyperbolic_circle(B,C);
hyperbolic_circle C2 = hyperbolic_circle(C,B);

// Intersect the two circles, resulting in the points I1, I2
hyperbolic_point[] I = intersection(C1,C2);

// Put a line L through the two intersection points I1,I2
hyperbolic_line SL3 = hyperbolic_line(I[0], I[1]);

// Intersect the line L with the segment S resulting in the midpoint M
hyperbolic_point MBC = intersection(SL3, L3);

// And draw all the stuff
draw(C1, gray+opacity(0.3));
draw(C2, gray+opacity(0.3));
draw(L3, gray+opacity(0.3));
dot(I[0], gray);
dot(I[1], gray);

//------------------------------------------------------------------------------
// The circumcircle center
hyperbolic_point CCC = intersection(SL1, SL2);

// The circumcircle
hyperbolic_circle CIRCIR = hyperbolic_circle(CCC, A);


// Draw
fill(T, blue+opacity(0.3));
draw(T);
dot(T);
dot(W, blue+opacity(0.5));
draw(SL1, blue+opacity(0.3));
draw(SL2, blue+opacity(0.3));
draw(SL3, blue+opacity(0.3));
dot(MAB, blue);
dot(MAC, blue);
dot(MBC, blue);
draw(CIRCIR, orange);
dot(CCC, orange);


draw((1,1)--(1,-1)--(-1,-1)--(-1,1)--cycle);
draw(unitcircle);
