import math;
import hyperbolic_geometry;

size(500,500);
settings.outformat="pdf";

// Here we show the construction of the midpoint of two points.
// This is done in analogy to the classical euclidean geometry.

hyperbolic_point O = hyperbolic_point(0,0);
dot(O,red);

// The two points A, B given
hyperbolic_point A = hyperbolic_point(1,20);
hyperbolic_point B = hyperbolic_point(1,140);

// A line through A, B
hyperbolic_line L = hyperbolic_line(A,B);

// Two circles with center {A,B} going through {B,A}
hyperbolic_circle C1 = hyperbolic_circle(A,B);
hyperbolic_circle C2 = hyperbolic_circle(B,A);

// Intersect the two circles, resulting in the points I1, I2
hyperbolic_point[] I = intersection(C1,C2);

// And the two other circles just for fun
hyperbolic_circle C3 = hyperbolic_circle(I[0],A);
hyperbolic_circle C4 = hyperbolic_circle(I[1],A);

// The two triangles
hyperbolic_triangle T1 = hyperbolic_triangle(A,B,I[0]);
hyperbolic_triangle T2 = hyperbolic_triangle(A,B,I[1]);

// And draw all the stuff
draw(C1, gray);
draw(C2, gray);
draw(C3, gray);
draw(C4, gray);
draw(L, black);
dot(A, black);
dot(B, black);
dot(I[0]);
dot(I[1]);

fill(T1, green+opacity(0.5));
fill(T2, blue+opacity(0.5));
draw(T1);
draw(T2);
dot(T1);
dot(T2);


// Some measurements

string s = string(T1.a(), 4);
label(s,(B.get_euclidean()+I[0].get_euclidean())/2);

string s = string(T1.b(), 4);
label(s,(A.get_euclidean()+I[0].get_euclidean())/2);

string s = string(T1.c(), 4);
label(s,(A.get_euclidean()+B.get_euclidean())/2);

string s = string(T1.alpha(), 4);
label(s,1.1*A.get_euclidean()+0.2*I[0].get_euclidean());

string s = string(T1.beta(), 4);
label(s,1.1*B.get_euclidean()+0.2*I[0].get_euclidean());

string s = string(T1.gamma(), 4);
label(s,1.1*I[0].get_euclidean());


// And for the other triangle
string s = string(T2.a(), 4);
label(s,(B.get_euclidean()+I[1].get_euclidean())/2);

string s = string(T2.b(), 4);
label(s,(A.get_euclidean()+I[1].get_euclidean())/2);

string s = string(T2.alpha(), 4);
label(s,1.1*A.get_euclidean()+0.1*I[1].get_euclidean());

string s = string(T2.beta(), 4);
label(s,1.1*B.get_euclidean()+0.1*I[1].get_euclidean());

string s = string(T2.gamma(), 4);
label(s,1.1*I[1].get_euclidean());


draw((1,1)--(1,-1)--(-1,-1)--(-1,1)--cycle);
draw(unitcircle);
