import math;
import hyperbolic_geometry;

size(500,500);
settings.outformat="pdf";

// Here we show the construction of the midpoint of two points.
// This is done in analogy to the classical euclidean geometry.

hyperbolic_point O = hyperbolic_point(0,0);
dot(O,red);

// The two points A, B given
hyperbolic_point A = hyperbolic_point(0.8,30);
hyperbolic_point B = hyperbolic_point(1.2,145);

// The line AB connection the A, B
hyperbolic_line AB = hyperbolic_line(A,B);

// Two circles with center {A,B} going through {B,A}
hyperbolic_circle C1 = hyperbolic_circle(A,B);
hyperbolic_circle C2 = hyperbolic_circle(B,A);

// Intersect the two circles, resulting in the points I1, I2
hyperbolic_point[] I = intersection(C1,C2);

// Put a line L through the two intersection points I1,I2
hyperbolic_line L = hyperbolic_line(I[0], I[1]);

// Intersect the line L with the segment S resulting in the midpoint M
hyperbolic_point M = intersection(AB, L);


// Normals
hyperbolic_line NA = hyperbolic_normal(AB,A);
hyperbolic_line NB = hyperbolic_normal(AB,B);

//
hyperbolic_circle D1 = hyperbolic_circle(A,M);
hyperbolic_circle D2 = hyperbolic_circle(B,M);

// Upper points
hyperbolic_point PNA = intersection(D1,NA)[0];
hyperbolic_point PNB = intersection(D2,NB)[1];

// Diagonal lines
hyperbolic_line GD1 = hyperbolic_line(PNA, B);
hyperbolic_line GD2 = hyperbolic_line(PNB, A);

// More circles
hyperbolic_circle E1 = hyperbolic_circle(PNA,A);
hyperbolic_circle E2 = hyperbolic_circle(PNB,B);

// Intersection points
hyperbolic_point X1 = intersection(E1, GD1)[0];
hyperbolic_point X2 = intersection(E2, GD2)[1];

// Circles
hyperbolic_circle F1 = hyperbolic_circle(A,X2);
hyperbolic_circle F2 = hyperbolic_circle(B,X1);

// And finally the golden ratio points
hyperbolic_point GR1 = intersection(F1,AB)[0];
hyperbolic_point GR2 = intersection(F2,AB)[1];


// And draw all the stuff
draw(AB, black);

draw(C1, gray+opacity(0.2));
draw(C2, gray+opacity(0.2));

draw(L, gray+opacity(0.2));

draw(NA, gray);
draw(NB, gray);

draw(D1, gray+opacity(0.3));
draw(D2, gray+opacity(0.3));

draw(E1, gray+opacity(0.4));
draw(E2, gray+opacity(0.4));

draw(GD1, blue+opacity(0.4));
draw(GD2, blue+opacity(0.4));

draw(F1, gray+opacity(0.5));
draw(F2, gray+opacity(0.5));





dot(A, red);
dot(B, black);

dot(I[0], gray);
dot(I[1], gray);

dot(M, gray);

dot(PNA, black);
dot(PNB, black);

dot(X1, gray);
dot(X2, gray);

dot(GR1, orange);
dot(GR2, red);

// Do some measurements
real dagr2 = hyperbolic_distance(A,GR2);
real dbgr1 = hyperbolic_distance(B,GR1);
real dgr1gr2 = hyperbolic_distance(GR1,GR2);
real dab = hyperbolic_distance(A,B);

string s = string(dagr2, 4);
label(s,(A.get_euclidean()+GR2.get_euclidean())/2);

string s = string(dbgr1, 4);
label(s,(B.get_euclidean()+GR1.get_euclidean())/2);

string s = string(dgr1gr2, 4);
label(s,(GR1.get_euclidean()+GR2.get_euclidean())/2);

string s = string(dab, 4);
label(s,(A.get_euclidean()+B.get_euclidean())/2+(0,.2));

// which show, that the golden ration does not hold!

draw((1,1)--(1,-1)--(-1,-1)--(-1,1)--cycle);
draw(unitcircle);
