import math;
import hyperbolic_geometry;

size(500,500);
settings.outformat="pdf";

// Here we show the construction of the midpoint of two points.
// This is done in analogy to the classical euclidean geometry.

hyperbolic_point O = hyperbolic_point(0,0);
dot(O,red);

// The two points A, B given
hyperbolic_point A = hyperbolic_point(2,60);
hyperbolic_point B = hyperbolic_point(0.5,120);

// The segment S connection the A, B
hyperbolic_segment S = hyperbolic_segment(A,B);

// Two circles with center {A,B} going through {B,A}
hyperbolic_circle C1 = hyperbolic_circle(A,B);
hyperbolic_circle C2 = hyperbolic_circle(B,A);

// Intersect the two circles, resulting in the points I1, I2
hyperbolic_point[] I = intersection(C1,C2);

// Put a line L through the two intersection points I1,I2
hyperbolic_line L = hyperbolic_line(I[0], I[1]);

// Intersect the line L with the segment S resulting in the midpoint M
hyperbolic_line s = hyperbolic_line(A,B);
hyperbolic_point M = intersection(s, L);

// And draw all the stuff
draw(C1, orange);
draw(C2, orange);
draw(L, gray);
draw(S, blue);
dot(A, black);
dot(B, black);
dot(I[0]);
dot(I[1]);
dot(M, blue);

// Do some measurements
real d1 = hyperbolic_distance(A,M);
real d2 = hyperbolic_distance(B,M);
real d = hyperbolic_distance(A,B);

string s = string(d, 4);
label(s,(A.get_euclidean()+B.get_euclidean())/2);

string s = string(d1, 4);
label(s,(A.get_euclidean()+M.get_euclidean())/2);

string s = string(d2, 4);
label(s,(M.get_euclidean()+B.get_euclidean())/2);


draw((1,1)--(1,-1)--(-1,-1)--(-1,1)--cycle);
draw(unitcircle);
