import math;
import hyperbolic_geometry;

size(500,500);
settings.outformat="pdf";

// Here we show the construction of a regular pentagon.
// This is done in analogy to the classical euclidean geometry.
// As we will see, this construction fails.

/* Description taken from: http://en.wikipedia.org/wiki/Pentagon
   0. Draw a circle in which to inscribe the pentagon and mark the center point O.
      (This is the green circle in the diagram to the right).
   1. Choose a point A on the circle that will serve as one vertex of the pentagon.
      Draw a line through O and A.
   2. Construct a line perpendicular to the line OA passing through O. Mark its
      intersection with one side of the circle as the point B.
   3. Construct the point C as the midpoint of O and B.
   4. Draw a circle centered at C through the point A. Mark its intersection with
      the line OB (inside the original circle) as the point D.
   5. Draw a circle centered at A through the point D. Mark its intersections with
      the original (green) circle as the points E and F.
   6. Draw a circle centered at E through the point A. Mark its other intersection
      with the original circle as the point G.
   7. Draw a circle centered at F through the point A. Mark its other intersection
      with the original circle as the point H.
   8. Construct the regular pentagon AEGHF.
*/

// Step 0, step 1
hyperbolic_point O = hyperbolic_point(1.0,80);
hyperbolic_point A = hyperbolic_point(2.0,20);
hyperbolic_line OA = hyperbolic_line(O,A);
hyperbolic_circle C = hyperbolic_circle(O,A);

// Step 2
hyperbolic_line N = hyperbolic_normal(OA, O);
hyperbolic_point B = intersection(C,N)[1];

// Step 3
hyperbolic_circle C1 = hyperbolic_circle(O,B);
hyperbolic_circle C2 = hyperbolic_circle(B,O);
hyperbolic_point[] I = intersection(C1,C2);
hyperbolic_line L = hyperbolic_line(I[0], I[1]);
hyperbolic_point M = intersection(N, L);

// Step 4
hyperbolic_circle C3 = hyperbolic_circle(M, A);
hyperbolic_point D = intersection(C3, hyperbolic_line(O,B))[0];

// Step 5
hyperbolic_circle C4 = hyperbolic_circle(A, D);
hyperbolic_point[] IP = intersection(C, C4);
hyperbolic_point E = IP[0];
hyperbolic_point F = IP[1];

// Step 6
hyperbolic_circle C5 = hyperbolic_circle(E, A);
hyperbolic_point G = intersection(C5, C)[1];

// Step 7
hyperbolic_circle C6 = hyperbolic_circle(F, A);
hyperbolic_point H = intersection(C6, C)[0];

// Step 8
hyperbolic_point[] EP;
EP[0] = A;
EP[1] = E;
EP[2] = G;
EP[3] = H;
EP[4] = F;

hyperbolic_polygon PG = hyperbolic_polygon(EP);
// And finished

// Label the important points
label("O",1.1*O.get_euclidean());
label("A",1.1*A.get_euclidean());
label("B",1.1*B.get_euclidean());
label("M",1.1*M.get_euclidean());
label("D",1.1*D.get_euclidean());
label("E",1.1*E.get_euclidean());
label("F",1.1*F.get_euclidean());
label("G",1.1*G.get_euclidean());
label("H",1.1*H.get_euclidean());

// And draw all the stuff
draw(OA, black);
draw(C, blue);
draw(N, gray);
//draw(C1, gray);
draw(C2, gray);
draw(L, gray);
draw(C3, gray);
draw(C4, gray);
draw(C5, gray);
draw(C6, gray);

fill(PG, orange+opacity(0.5));
draw(PG, red);
dot(PG, red);

dot(O, black);
dot(A, black);
dot(B, gray);
dot(I, gray);
dot(M, gray);
dot(D, gray);
dot(E, gray);
dot(F, gray);
dot(G, orange);
dot(H, orange);

// Do some measurements whoch show that the regular pentagon is not that regular
hyperbolic_segment[] side;
side[0] = hyperbolic_segment(EP[0], EP[1]);
side[1] = hyperbolic_segment(EP[1], EP[2]);
side[2] = hyperbolic_segment(EP[2], EP[3]);
side[3] = hyperbolic_segment(EP[3], EP[4]);
side[4] = hyperbolic_segment(EP[4], EP[0]);

for (int i = 0; i <= 4; ++i) {
    string s = string(side[i].length(), 4);
    label(s,(side[i].get_first_point().get_euclidean()+side[i].get_second_point().get_euclidean())/2);
}

for (int i = 0; i <= 4; ++i) {
    string s = string(angle(EP[(i)%5], EP[(i+1)%5], EP[(i+2)%5]), 4);
    label(s,EP[(i+1)%5].get_euclidean());
}

draw((1,1)--(1,-1)--(-1,-1)--(-1,1)--cycle);
draw(unitcircle);
