import math;
import hyperbolic_geometry;

size(500,500);
settings.outformat="pdf";

// What do we want to test?
// Mirroring hyperbolic rays at hyperbolic segments. The mirror
// sequence is gray to orange to yellow to red

hyperbolic_point A = hyperbolic_point(1,90);
hyperbolic_point B = hyperbolic_point(1,30);
hyperbolic_segment M = hyperbolic_segment(A,B);
draw(M, blue);
dot(A, blue);
dot(B, blue);

hyperbolic_point C = hyperbolic_point(3, 40+180);
hyperbolic_point D = hyperbolic_point(3, 220+180);
hyperbolic_segment N = hyperbolic_segment(C, D);
draw(N, blue);
dot(C, blue);
dot(D, blue);

hyperbolic_point E = hyperbolic_point(1, 120);
hyperbolic_point F = hyperbolic_point(0.5, 245);
hyperbolic_segment O = hyperbolic_segment(E, F);
draw(O, blue);
dot(E, blue);
dot(F, blue);

hyperbolic_point[] P;
hyperbolic_point[] Q;

P[0] = hyperbolic_point(2, 45);
Q[0] = hyperbolic_point(1.2, 10);
P[1] = hyperbolic_point(1.2, 120);
Q[1] = hyperbolic_point(1.2, 10);
P[2] = hyperbolic_point(3,45);
Q[2] = hyperbolic_point(2.5,60);
P[3] = hyperbolic_point(3,90);
Q[3] = hyperbolic_point(3.5,100);
P[4] = hyperbolic_point(4,70);
Q[4] = hyperbolic_point(4.5,72);
P[5] = hyperbolic_point(1,280);
Q[5] = hyperbolic_point(1,290);

hyperbolic_ray G;
hyperbolic_ray H;
hyperbolic_ray I;
hyperbolic_ray J;

hyperbolic_point R;
hyperbolic_point S;
hyperbolic_point T;

hyperbolic_segment Seg;

for(int i=0; i < P.length; ++i) {
    dot(P[i], gray);
    dot(Q[i], gray);

    G = hyperbolic_ray(P[i], Q[i]);
    H = mirror(M, G);
    I = mirror(N, H);
    J = mirror(O, I);

    R = mirror(M, Q[i]);
    S = mirror(N, R);
    T = mirror(O, S);

    draw(G, gray);
    draw(H, orange);
    draw(I, yellow);
    draw(J, red);
    dot(G.get_point(), gray);
    dot(G.get_endpoint(), black);
    dot(H.get_point(), orange);
    dot(H.get_endpoint(), black);
    dot(I.get_point(), yellow);
    dot(I.get_endpoint(), black);
    dot(J.get_point(), red);
    dot(J.get_endpoint(), black);

    dot(R, orange);
    dot(S, yellow);
    dot(T, red);

    // Track starting points
    Seg = hyperbolic_segment(G.get_point(), H.get_point());
    draw(Seg, blue+opacity(0.2));
    Seg = hyperbolic_segment(H.get_point(), I.get_point());
    draw(Seg, blue+opacity(0.2));
    Seg = hyperbolic_segment(I.get_point(), J.get_point());
    draw(Seg, blue+opacity(0.2));

    // Track ideal points
//     draw(G.get_endpoint()--H.get_endpoint(), blue+opacity(0.4));
//     draw(H.get_endpoint()--I.get_endpoint(), blue+opacity(0.4));
//     draw(I.get_endpoint()--J.get_endpoint(), blue+opacity(0.4));

}

draw((1,1)--(1,-1)--(-1,-1)--(-1,1)--cycle);
draw(unitcircle);
